using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Monitor;
using Plux;

[Extension("Monitor", OnCreated="OnCreated", OnReleased="OnReleased")]
[Plug("Startup")]
[ParamValue("ExecuteInMainThread", true)]
[Monitor(OnExtensionCreated = "ExtensionCreated", OnExtensionReleased = "ExtensionReleased")]
public partial class PluginView : Form, IStartup {
  public PluginView() {
    InitializeComponent();
    MAX_VISIBLESLOTS = (int) Math.Floor(
                               (Screen.GetWorkingArea(this).Height - 212)
                               /(double) _dummy.Height);
    Visible = false;
  }
  public void Run() {}
  public void OnCreated(object sender, ExtensionEventArgs e) {
    Show();
  }
  public void OnReleased(object source, ExtensionEventArgs e) {
    Hide();
  }

  public void ExtensionCreated(object source, ExtensionEventArgs e) {
    if (_slots.ContainsKey(e.ExtensionInfo)) {
      _slots[e.ExtensionInfo].AddRef();
      return;
    }
    SlotButton l = new SlotButton(e.ExtensionInfo);
    if(Controls.Count > 0) {
      Control pred = Controls[Controls.Count - 1];
      l.Location = new Point(0, pred.Location.Y + _dummy.Height);
    }
    Controls.Add(l);
    _slots.Add(e.ExtensionInfo, l);

    if(_slots.Count > 1 && _slots.Count <= MAX_VISIBLESLOTS)
      Height += _dummy.Height;

    Visible = _slots.Count > 0;
    SetWidth();

    Update();
  }
  public void ExtensionReleased(object source, ExtensionEventArgs e) {
    if(!Visible 
       || !_slots.ContainsKey(e.ExtensionInfo) 
       || !_slots[e.ExtensionInfo].Release())
      return;

    UserControl ctl = _slots[e.ExtensionInfo];
    ctl.Visible = false;
    int idx = Controls.IndexOf(ctl);
    Controls.Remove(ctl);
    _slots.Remove(e.ExtensionInfo);

    for(; idx < Controls.Count; ++idx) {
      Control succ = Controls[idx];
      succ.Location = new Point(0, succ.Location.Y - _dummy.Height);
    }

    if(_slots.Count > 0 && _slots.Count < MAX_VISIBLESLOTS)
      Height -= _dummy.Height;

    Visible = _slots.Count > 0;
    SetWidth();

    Update();
  }
  void SetWidth() {
    if(_slots.Count > MAX_VISIBLESLOTS) {
      Width = _dummy.Width + new VScrollBar().Width;
      HScroll = false;
      AutoScroll = true;
    }
    else {
      Width = _dummy.Width;
      AutoScroll = false;
    }
  }

  // Hide Borderless Form from Alt+Tab
  protected override CreateParams CreateParams {
    get {
      CreateParams cp = base.CreateParams;
      // turn on WS_EX_TOOLWINDOW style bit
      cp.ExStyle |= 0x80;
      return cp;
    }
  }

  //IRepository _repository;
  Dictionary<ExtensionInfo, SlotButton> _slots =
    new Dictionary<ExtensionInfo, SlotButton>();
  SlotButton _dummy = new SlotButton();
  readonly int MAX_VISIBLESLOTS;
}