using System;
using System.Collections.Generic;

namespace Plux {
  public delegate string CommandHandler(string[] args);

  [SlotDefinition("Commands")]
  public interface IConsoleCommands {
    IEnumerable<ICommand> Commands { get; }
  }

  public interface ICommand {
    string Name { get; }
    CommandHandler Handler { get; }
    string Help { get; }
    IEnumerable<string> GetArguments(string[] command, int pos);
  }

  [SlotDefinition("Plux.CommandInterpreter")]
  public interface ICommandInterpreter {
    event EventHandler StateChanged;
    CommandState State { get; }
    string Output { get; }
    string Execute(string command);
    bool ExecuteAsync(string command);
    string[] GetCommandOptions(ref string cmdStart);
  }

  public enum CommandState {
    UnInititalized,
    Completed,
    Running,
    Failed,
    Stopped
  }

  public class CommandExecutionException : Exception
  {
      public object[] ErrorList;

      public CommandExecutionException(string error)
      {
          ErrorList = new object[] { error };
      }
  }
}