using System;
using System.Collections.Generic;
using System.Text;


internal static class HTMLCharacterConverter {
    private static readonly Dictionary<char, string> htmlEscapeLiterals =
            new Dictionary<char, string>();

    static HTMLCharacterConverter() {
        htmlEscapeLiterals.Add('"', "&quot;");
        htmlEscapeLiterals.Add('<', "&lt;");
        htmlEscapeLiterals.Add('>', "&gt;");
        htmlEscapeLiterals.Add('', "&iexcl;");
        htmlEscapeLiterals.Add('', "&cent;");
        htmlEscapeLiterals.Add('', "&pound;");
        htmlEscapeLiterals.Add('', "&curren;");
        htmlEscapeLiterals.Add('', "&yen;");
        htmlEscapeLiterals.Add('', "&brvbar;");
        htmlEscapeLiterals.Add('', "&sect;");
        htmlEscapeLiterals.Add('', "&uml;");
        htmlEscapeLiterals.Add('', "&copy;");
        htmlEscapeLiterals.Add('', "&ordf;");
        htmlEscapeLiterals.Add('', "&laquo;");
        htmlEscapeLiterals.Add('', "&not;");
        htmlEscapeLiterals.Add('', "&shy;");
        htmlEscapeLiterals.Add('', "&reg;");
        htmlEscapeLiterals.Add('', "&macr;");
        htmlEscapeLiterals.Add('', "&deg;");
        htmlEscapeLiterals.Add('', "&plusmn;");
        htmlEscapeLiterals.Add('', "&sup2;");
        htmlEscapeLiterals.Add('', "&sup3;");
        htmlEscapeLiterals.Add('', "&acute;");
        htmlEscapeLiterals.Add('', "&micro;");
        htmlEscapeLiterals.Add('', "&para;");
        htmlEscapeLiterals.Add('', "&middot;");
        htmlEscapeLiterals.Add('', "&cedil;");
        htmlEscapeLiterals.Add('', "&sup1;");
        htmlEscapeLiterals.Add('', "&ordm;");
        htmlEscapeLiterals.Add('', "&raquo;");
        htmlEscapeLiterals.Add('', "&frac14;");
        htmlEscapeLiterals.Add('', "&frac12;");
        htmlEscapeLiterals.Add('', "&frac34;");
        htmlEscapeLiterals.Add('', "&iquest;");
        htmlEscapeLiterals.Add('', "&Agrave;");
        htmlEscapeLiterals.Add('', "&Aacute;");
        htmlEscapeLiterals.Add('', "&Acirc;");
        htmlEscapeLiterals.Add('', "&Atilde;");
        htmlEscapeLiterals.Add('', "&Auml;");
        htmlEscapeLiterals.Add('', "&Aring;");
        htmlEscapeLiterals.Add('', "&AElig;");
        htmlEscapeLiterals.Add('', "&Ccedil;");
        htmlEscapeLiterals.Add('', "&Egrave;");
        htmlEscapeLiterals.Add('', "&Eacute;");
        htmlEscapeLiterals.Add('', "&Ecirc;");
        htmlEscapeLiterals.Add('', "&Euml;");
        htmlEscapeLiterals.Add('', "&Igrave;");
        htmlEscapeLiterals.Add('', "&Iacute;");
        htmlEscapeLiterals.Add('', "&Icirc;");
        htmlEscapeLiterals.Add('', "&Iuml;");
        htmlEscapeLiterals.Add('', "&ETH;");
        htmlEscapeLiterals.Add('', "&Ntilde;");
        htmlEscapeLiterals.Add('', "&Ograve;");
        htmlEscapeLiterals.Add('', "&Oacute;");
        htmlEscapeLiterals.Add('', "&Ocirc;");
        htmlEscapeLiterals.Add('', "&Otilde;");
        htmlEscapeLiterals.Add('', "&Ouml;");
        htmlEscapeLiterals.Add('', "&times;");
        htmlEscapeLiterals.Add('', "&Oslash;");
        htmlEscapeLiterals.Add('', "&Ugrave;");
        htmlEscapeLiterals.Add('', "&Uacute;");
        htmlEscapeLiterals.Add('', "&Ucirc;");
        htmlEscapeLiterals.Add('', "&Uuml;");
        htmlEscapeLiterals.Add('', "&Yacute;");
        htmlEscapeLiterals.Add('', "&THORN;");
        htmlEscapeLiterals.Add('', "&szlig;");
        htmlEscapeLiterals.Add('', "&agrave;");
        htmlEscapeLiterals.Add('', "&aacute;");
        htmlEscapeLiterals.Add('', "&acirc;");
        htmlEscapeLiterals.Add('', "&atilde;");
        htmlEscapeLiterals.Add('', "&auml;");
        htmlEscapeLiterals.Add('', "&aring;");
        htmlEscapeLiterals.Add('', "&aelig;");
        htmlEscapeLiterals.Add('', "&ccedil;");
        htmlEscapeLiterals.Add('', "&egrave;");
        htmlEscapeLiterals.Add('', "&eacute;");
        htmlEscapeLiterals.Add('', "&ecirc;");
        htmlEscapeLiterals.Add('', "&euml;");
        htmlEscapeLiterals.Add('', "&igrave;");
        htmlEscapeLiterals.Add('', "&iacute;");
        htmlEscapeLiterals.Add('', "&icirc;");
        htmlEscapeLiterals.Add('', "&iuml;");
        htmlEscapeLiterals.Add('', "&eth;");
        htmlEscapeLiterals.Add('', "&ntilde;");
        htmlEscapeLiterals.Add('', "&ograve;");
        htmlEscapeLiterals.Add('', "&oacute;");
        htmlEscapeLiterals.Add('', "&ocirc;");
        htmlEscapeLiterals.Add('', "&otilde;");
        htmlEscapeLiterals.Add('', "&ouml;");
        htmlEscapeLiterals.Add('', "&divide;");
        htmlEscapeLiterals.Add('', "&oslash;");
        htmlEscapeLiterals.Add('', "&ugrave;");
        htmlEscapeLiterals.Add('', "&uacute;");
        htmlEscapeLiterals.Add('', "&ucirc;");
        htmlEscapeLiterals.Add('', "&uuml;");
        htmlEscapeLiterals.Add('', "&yacute;");
        htmlEscapeLiterals.Add('', "&thorn;");
        htmlEscapeLiterals.Add('', "&yuml;");
    }

    public static String CreateHtmlSaveStringFrom(String text) {
        if (String.IsNullOrEmpty(text)) {
            return String.Empty;
        }
        StringBuilder cleanText = new StringBuilder();
        foreach (char c in text) {
            if (htmlEscapeLiterals.ContainsKey(c)) {
                cleanText.Append(htmlEscapeLiterals[c]);
            } else {
                cleanText.Append(c);
            }
        }
        return cleanText.ToString();
    }
}