using System;
using System.Collections.Generic;

namespace Plux.Framework {
  [AttributeUsage(AttributeTargets.Class, AllowMultiple = true, Inherited = true)]
  public class ReplaceSlotAttribute : FrameworkSlotAttribute {
    public bool Register = false;
    public bool Plug = false;
    public bool Global = false;
    public ReplaceSlotAttribute(string slot) : base(slot) {}
  }

  public class ReplaceRegister {
    public SlotInfo slot;
    internal bool global;
    public ReplaceRegister(SlotInfo slot) {
      this.slot = slot;
      slot.Registering += Registering;
    }
    public ReplaceRegister(SlotInfo slot, bool global)
      : this(slot) {
      this.global = global;
    }

    void Registering(object sender, CancelRegisterEventArgs args) {
      if (global) {
        SlotInfo[] slots = Runtime.Repository.SlotInfos.Find(slot.Name);
        foreach (SlotInfo si in slots)
          foreach (PlugTypeInfo pti in si.RegisteredPlugTypeInfos)
            if (pti != args.PlugTypeInfo) si.Deregister(pti);
        args.PlugTypeInfo.Register();
      } else {
        foreach (PlugTypeInfo plugType in slot.RegisteredPlugTypeInfos)
          if (plugType != args.PlugTypeInfo) slot.Deregister(plugType);
      }
    }
  }

  class ReplaceRegisterStartup : SlotStartup<ReplaceSlotAttribute> {
    static readonly Dictionary<SlotInfo, ReplaceRegister> data =
      new Dictionary<SlotInfo, ReplaceRegister>();
    public override void Register(ExtensionTypeInfo extType) {}
    public override void Unregister(ExtensionTypeInfo extType) {}
    public override void SlotCreated(SlotInfo slot, ReplaceSlotAttribute attr) {
      if (attr.Register)
        data[slot] = new ReplaceRegister(slot, attr.Global);
    }
    public override void SlotReleased(SlotInfo slot, ReplaceSlotAttribute attr) {
      if (attr.Register)
        data.Remove(slot);
    }
  }
}