using System;
using System.IO;
using System.Reflection;

namespace Plux.Framework {
  public sealed class ResourceStreamGetter : IDisposable {
    private Stream _stream;
    
    public ResourceStreamGetter(PluginInfo plugin, string name,
                                bool useNamespace) {
      if(plugin == null)
        throw new ArgumentNullException("plugin");
      if(name == null)
        throw new ArgumentNullException("name");

      Assembly asm = plugin.Assembly;
      string res;
      if(useNamespace) {
        string file = System.IO.Path.GetFileNameWithoutExtension(plugin.Name);
        res = String.Format("{0}.{1}", file, name);
      }
      else res = name;
      _stream = asm.GetManifestResourceStream(res);
    }
    public Stream Stream {
      get { return _stream; }
    }
    #region IDisposable Members
    public void Dispose() {
      Dispose(true);
      GC.SuppressFinalize(this);
    }
    #endregion
    ~ResourceStreamGetter() {
      Dispose(false);
    }
    private void Dispose(bool disposing) {
      if(disposing)
        if(_stream != null) {
          _stream.Dispose();
          _stream = null;
        }
    }
    public static ResourceStreamGetter GetResourceStream
      (PluginInfo plugin, string resourceName, bool useNamespace) {
      return new ResourceStreamGetter(plugin, resourceName, useNamespace);
    }
  }
}