using System;
using System.Collections.Generic;
using Plux;

namespace Plux.Framework {

  [Extension]
  [Plug("Commands")]
  public class RuntimeCommands : IConsoleCommands {
    private readonly List<ICommand> commands = new List<ICommand>();
    internal static readonly string lf = Environment.NewLine;

    public RuntimeCommands() {
      commands.Add(new ListCommand());
      commands.Add(new FindCommand());
      commands.Add(new RegisterCommand());
      commands.Add(new UnregisterCommand());
      commands.Add(new CreateCommand());
      commands.Add(new PlugCommand());
      commands.Add(new UnplugCommand());
      commands.Add(new ShutdownCommand());
      commands.Add(new ReleaseCommand());
      commands.Add(new PropertyCommand());
      commands.Add(new UptimeCommand());
      commands.Add(new CallCommand());
      commands.Add(new OpenCommand());
      commands.Add(new CloseCommand());
      commands.Add(new SelectCommand());
      commands.Add(new SetCommand());
    }

    public IEnumerable<ICommand> Commands {
      get { return commands; }
    }

    public static RepositoryElement[] Find(
        string[] command, int index, out string message) {
      message = String.Empty;
      if (command.Length < index + 2) return new RepositoryElement[0];
      int id;
      string filter = command[index + 1];
      filter.Trim(new char[] {'\"', '\''});
      switch (command[index]) {
        case "PluginInfo":
          if (filter == "*")
            return Runtime.Repository.PluginInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            PluginInfo pi = Runtime.Repository.PluginInfos.Find(id);
            if (pi == null) {
              message =
                  String.Format(
                      "PluginInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {pi};
          } else {
            PluginInfo[] piArr = Runtime.Repository.PluginInfos.Find(filter);
            if (piArr.Length == 0) {
              message =
                  String.Format(
                      "PluginInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return piArr;
          }
          break;

        case "ExtensionTypeInfo":
          if (filter == "*")
            return Runtime.Repository.ExtensionTypeInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            ExtensionTypeInfo eti =
                Runtime.Repository.ExtensionTypeInfos.Find(id);
            if (eti == null) {
              message =
                  String.Format(
                      "ExtensionTypeInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {eti};
          } else {
            ExtensionTypeInfo[] etiArr =
                Runtime.Repository.ExtensionTypeInfos.Find(filter);
            if (etiArr.Length == 0) {
              message =
                  String.Format(
                      "ExtensionTypeInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return etiArr;
          }
          break;

        case "PlugTypeInfo":
          if (filter == "*")
            return Runtime.Repository.PlugTypeInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            PlugTypeInfo pti = Runtime.Repository.PlugTypeInfos.Find(id);
            if (pti == null) {
              message =
                  String.Format(
                      "PlugTypeInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {pti};
          } else {
            PlugTypeInfo[] ptiArr =
                Runtime.Repository.PlugTypeInfos.Find(filter);
            if (ptiArr.Length == 0) {
              message =
                  String.Format(
                      "PlugTypeInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return ptiArr;
          }
          break;

        case "SlotTypeInfo":
          if (filter == "*")
            return Runtime.Repository.SlotTypeInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            SlotTypeInfo sti = Runtime.Repository.SlotTypeInfos.Find(id);
            if (sti == null) {
              message =
                  String.Format(
                      "SlotTypeInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {sti};
          } else {
            SlotTypeInfo[] stiArr =
                Runtime.Repository.SlotTypeInfos.Find(filter);
            if (stiArr.Length == 0) {
              message =
                  String.Format(
                      "SlotTypeInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return stiArr;
          }
          break;

        case "ExtensionInfo":
          if (filter == "*")
            return Runtime.Repository.ExtensionInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            ExtensionInfo ei = Runtime.Repository.ExtensionInfos.Find(id);
            if (ei == null) {
              message =
                  String.Format(
                      "ExtensionInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {ei};
          } else {
            ExtensionInfo[] eiArr =
                Runtime.Repository.ExtensionInfos.Find(filter);
            if (eiArr.Length == 0) {
              message =
                  String.Format(
                      "ExtensionInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return eiArr;
          }
          break;

        case "Plug":
          if (filter == "*")
            return Runtime.Repository.PlugInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            PlugInfo pi = Runtime.Repository.PlugInfos.Find(id);
            if (pi == null) {
              message =
                  String.Format(
                      "Plug with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {pi};
          } else {
            PlugInfo[] piArr = Runtime.Repository.PlugInfos.Find(filter);
            if (piArr.Length == 0) {
              message =
                  String.Format(
                      "Plug with the name '{0}' not found.",
                      command[index + 1]);
            }
            return piArr;
          }
          break;

        case "SlotInfo":
          if (filter == "*")
            return Runtime.Repository.SlotInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            SlotInfo si = Runtime.Repository.SlotInfos.Find(id);
            if (si == null) {
              message =
                  String.Format(
                      "SlotInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {si};
          } else {
            SlotInfo[] siArr = Runtime.Repository.SlotInfos.Find(filter);
            if (siArr.Length == 0) {
              message =
                  String.Format(
                      "SlotInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return siArr;
          }
          break;

        case "ContractInfo":
          if (filter == "*")
            return Runtime.Repository.ContractInfos.ToArray();
          if (Int32.TryParse(filter, out id)) {
            ContractInfo ci = Runtime.Repository.ContractInfos.Find(id);
            if (ci == null) {
              message =
                  String.Format(
                      "ContractInfo with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {ci};
          } else {
            ContractInfo[] ciArr = Runtime.Repository.ContractInfos.Find(filter);
            if (ciArr.Length == 0) {
              message =
                  String.Format(
                      "ContractInfo with the name '{0}' not found.",
                      command[index + 1]);
            }
            return ciArr;
          }
          break;

        case "SlotDefinition":
          if (filter == "*")
            return Runtime.Repository.SlotDefinitions.ToArray();
          if (Int32.TryParse(filter, out id)) {
            SlotDefinition sd = Runtime.Repository.SlotDefinitions.Find(id);
            if (sd == null) {
              message =
                  String.Format(
                      "SlotDefinition with the id {0} not found.", id);
            } else
              return new RepositoryElement[] {sd};
          } else {
            SlotDefinition[] sdArr =
                Runtime.Repository.SlotDefinitions.Find(filter);
            if (sdArr.Length == 0) {
              message =
                  String.Format(
                      "SlotDefinition with the name '{0}' not found.",
                      command[index + 1]);
            }
            return sdArr;
          }
          break;
      }
      return new RepositoryElement[0];
    }

    internal static bool ContainsArgument(string value, string[] args) {
      foreach (string s in args) {
        if (value == s)
          return true;
      }
      return false;
    }
  }
}
