using System;
using System.ComponentModel;
using System.Windows.Forms;
using Metrix.Contracts;
using Plux;

[Extension]
[Plug("Workbench.Sidebar")]
[ParamValue("Tag", "WORKBENCH~SIDEBAR")]
[Slot("Plux.Figure", AutoPlug = false, Multiple=true,
  OnRegistered = "Figure_Registered", OnPlugged = "Figure_Plugged")]
internal class Sidebar : IControl {
  private readonly Panel panel = new Panel();
  private readonly Usage usage1;
  private readonly Usage usage2;
  private readonly Usage2 usage3;
  private IMeasurement registeredTypeCount = null;
  private IMeasurement bytesInUse = null;
  private IMeasurement extensionCount = null;
  //private IMeasurement instancedTypeCount = null;
  private float max = 0f;
  private IMeasurement typeCount = null;
  private float typesInRepository = 0;
  public Sidebar() {
    panel.Dock = DockStyle.Fill;
    usage1 = new Usage();
    usage1.Margin = new Padding(10);
    usage1.Caption = "Bytes in use";
    usage2 = new Usage();
    panel.Width = Math.Max(usage1.Width, usage2.Width);
    usage2.Caption = "#Extensions";
    usage3 = new Usage2();
    usage3.Caption = "#Types";
    usage1.Dock = DockStyle.Top;
    usage2.Dock = DockStyle.Top;
    usage3.Dock = DockStyle.Top;
    panel.Controls.Add(usage3);
    panel.Controls.Add(usage2);
    panel.Controls.Add(usage1);
  }
  #region IControl Members
  public Control Control {
    get { return panel; }
  }
  #endregion
  public void Figure_Registered(object s, RegisterEventArgs args) {
    string name = (string) args.GetParamValue("FigureName");
    string scope = (string) args.GetParamValue("Scope");
    var figureSlot = Runtime.GetSlotInfo(this, "Plux.Figure");
    string[,] figures = {
                          {"bytesinuse", "Runtime"},
                          {"extensioninfos", "Runtime"},
                          {"extensiontypeinfos", "Runtime"},
                          {"registeredextensiontypeinfos", "Runtime"},
                          {"instancedextensiontypeinfos", "Runtime"}
                        };
    for(int i = 0; i <= figures.GetUpperBound(0); i++) {
      if (!name.Equals(figures[i, 0])) continue;
      if (!scope.Equals(figures[i, 1])) continue;
      figureSlot.Plug(args.PlugTypeInfo);
      //ExtensionInfo ext =
      //  args.PlugTypeInfo.ExtensionTypeInfo.GetSharedExtension(true);
      //ext.PlugPlugs();
    }
  }
  public void Figure_Plugged(object s, PlugEventArgs args) {
    IFigure f = (IFigure) args.Extension;
    if (args.GetParamValue("FigureName").Equals("bytesinuse")) {
      bytesInUse = f[null];
      SamplingMetrixDataItem item =
        new SamplingMetrixDataItem(f[null], "Memory", false, 1000);
      item.Kilo = true;
      item.PropertyChanged += OnPropertyChanged1;
    }
    if (args.GetParamValue("FigureName").Equals("extensioninfos")) {
      extensionCount = f[null];
      extensionCount.PropertyChanged += OnPropertyChanged2;
      OnPropertyChanged2(null, null);
    }
    if (args.GetParamValue("FigureName").Equals("extensiontypeinfos")) {
      typeCount = f[null];
      typeCount.PropertyChanged += OnPropertyChanged3;
      OnPropertyChanged3(null, null);
    }
    if (args.GetParamValue("FigureName").Equals("registeredextensiontypeinfos")) {
      registeredTypeCount = f[null];
      registeredTypeCount.PropertyChanged += OnPropertyChanged4;
      OnPropertyChanged4(null, null);
    }
    //if (args.GetParamValue("FigureName").Equals("instancedextensiontypeinfos")) {
    //  instancedTypeCount = f[Runtime.Repository.ExtensionInfos.Find("Runtime")[0]];
    //  instancedTypeCount.PropertyChanged += OnPropertyChanged4;
    //  OnPropertyChanged4(null, null);
    //}
  }
  private void OnPropertyChanged1(object sender, PropertyChangedEventArgs e) {
    if (usage1.InvokeRequired)
      usage1.BeginInvoke(new NoParameterDelegate(UpdateMemoryUsage));
    else
      UpdateMemoryUsage();
  }
  private void UpdateMemoryUsage() {
    float current = (long) bytesInUse.Current;
    usage1.Digit.Text = String.Format("{0,5}", (long) bytesInUse.Current/1024);
    max = Math.Max(current, max);
    usage1.Bar.Value = (max != 0) ? current/max : 0f;
    usage1.Invalidate();
    OnPropertyChanged4(null, null);
  }
  private void OnPropertyChanged2(object sender, PropertyChangedEventArgs e) {
    usage2.Digit.Text = String.Format("{0,5}", (int) extensionCount.Current);
    float current = (int) extensionCount.Current;
    float total = extensionCount.Total;
    usage2.Bar.Value = (total != 0) ? current/total : 0f;
    usage2.Invalidate();
  }
  private void OnPropertyChanged3(object sender, PropertyChangedEventArgs e) {
    typesInRepository = (int) typeCount.Current;
    OnPropertyChanged4(null, null);
  }
  private void OnPropertyChanged4(object sender, PropertyChangedEventArgs e) {
    int cntPlugged = 0/*, cntRegistered = 0*/;
    foreach (ExtensionTypeInfo type in Runtime.Repository.ExtensionTypeInfos) {
      if (type.ExtensionInfos.Count > 0) cntPlugged++;
      //foreach (PlugTypeInfo plugTypeInfo in type.PlugTypeInfos) {
      //  if (plugTypeInfo.RegisteredInSlots.Count > 0) {
      //    cntRegistered++;
      //    break;
      //  }
      //}
    }
    if (registeredTypeCount != null) {
      usage3.Digit1.Text = String.Format("{0,4}", (int) registeredTypeCount.Current);
      usage3.Bar1.Value = (typesInRepository != 0)
                            ? (int)registeredTypeCount.Current / typesInRepository
                            : 0f;
    }
    //if(instancedTypeCount != null) {
      usage3.Digit2.Text = String.Format("{0,4}", cntPlugged/*(int) instancedTypeCount.Current*/);
      usage3.Bar2.Value = (typesInRepository != 0)
                            ? cntPlugged/*(int)instancedTypeCount.Current*/ / typesInRepository
                            : 0f;
    //}
    usage3.Invalidate();
  }
  #region Nested type: NoParameterDelegate
  private delegate void NoParameterDelegate();
  #endregion
}