using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using Metrix.Contracts;
using Plux;

[Extension]
[Plug("Plux.Control", AutoRegister = false)]
[Slot("Plux.Figure", Unique=false, AutoRegister=false,
  OnPlugged="Figure_Plugged")]
public partial class Usage : UserControl, IControl {
  private IMeasurement measure = null;
  private float max = 0f;
  private int factor = 1;
  public Usage() {
    InitializeComponent();
  }
  public Control Control {
    get { return this; }
  }
  public void Figure_Plugged(object s, PlugEventArgs args) {
    IFigure f = (IFigure) args.Extension;
    caption.Text = (string) args.GetParamValue("FigureName");
    measure = f[null];
    if (args.GetParamValue("FigureName").Equals("bytesinuse")) {
      SamplingMetrixDataItem item =
        new SamplingMetrixDataItem(f[null], "Memory", false, 1000);
      item.Kilo = true;
      item.PropertyChanged += OnValueChanged;
    } else {
      measure.PropertyChanged += OnValueChanged;
      OnValueChanged(null, null);
    }
  }
  private delegate void NoParameterDelegate();
  private void OnValueChanged(object sender, PropertyChangedEventArgs e) {
    if (InvokeRequired)
      BeginInvoke(new NoParameterDelegate(UpdateMemoryUsage));
    else
      UpdateMemoryUsage();
  }
  private void UpdateMemoryUsage() {
    float current;
    if(measure.Current.GetType() == typeof(long))
      current = (long) measure.Current;
    else 
      current = (int) measure.Current;
    Digit.Text = String.Format("{0,5}", (int) current/Factor);
    max = Math.Max(current, max);
    Bar.Value = (max != 0) ? current/max : 0f;
    Invalidate();
  }
  [Browsable(true)]
  [Category("Appearance")]
  [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
  public string Caption {
    get { return caption.Text; }
    set { caption.Text = value; }
  }
  public int Factor {
    get { return factor; }
    set { factor = value; }
  }
  public override Size GetPreferredSize(Size proposedSize) {
    return Size;
  }
}